"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChartTile = void 0;
const react_1 = __importDefault(require("react"));
const react_native_1 = require("react-native");
const ui_1 = require("@cs-bosch-medusa/ui");
const chart_tile_label_1 = require("./chart-tile-label");
const theme_1 = require("../../theme");
const styles = react_native_1.StyleSheet.create({
    container: {
        flex: 1,
    },
    labelContainer: {
        flex: 4,
        justifyContent: "flex-end",
        paddingBottom: 2,
        paddingHorizontal: ui_1.METRICS.CONTAINER.halfBasePadding,
        paddingTop: ui_1.METRICS.CONTAINER.halfBasePadding,
    },
    middleContainer: {
        /**
         * NOTE: there was a request from the client that
         * the value labels should be aligned in one line
         * across the tiles.
         */
        flex: 5,
        justifyContent: "flex-end",
        paddingHorizontal: ui_1.METRICS.CONTAINER.halfBasePadding,
        paddingTop: ui_1.METRICS.CONTAINER.halfBasePadding,
    },
    middleRowContainer: {
        flexDirection: "row",
        justifyContent: "center",
    },
    unit: {
        paddingLeft: 2,
    },
    value: {
        paddingTop: ui_1.METRICS.CONTAINER.halfBasePadding,
    },
});
const ChartTile = (props) => {
    const { chartTheme } = (0, theme_1.useChartTheme)();
    const { value, unit, label, color, activated = true, } = props;
    const tileColor = activated ? color : chartTheme.tile.deactivatedColor;
    const labelColor = activated
        ? chartTheme.tile.labelColor
        : chartTheme.tile.deactivatedColor;
    return (react_1.default.createElement(react_native_1.View, { style: [
            styles.container,
            {
                backgroundColor: chartTheme.tile.backgroundColor,
                borderColor: tileColor,
                borderTopWidth: chartTheme.tile.topLineSize,
            },
        ] },
        react_1.default.createElement(react_native_1.View, { style: styles.middleContainer },
            react_1.default.createElement(react_native_1.View, { style: styles.middleRowContainer },
                react_1.default.createElement(ui_1.Label, { style: [
                        styles.value,
                        {
                            color: tileColor,
                        },
                    ] }, value),
                react_1.default.createElement(chart_tile_label_1.ChartTileLabel, { style: [styles.unit, { color: tileColor }] }, unit))),
        react_1.default.createElement(react_native_1.View, { style: styles.labelContainer },
            react_1.default.createElement(chart_tile_label_1.ChartTileLabel, { style: { color: labelColor } }, label))));
};
exports.ChartTile = ChartTile;
