export type ChartSelectionProps = {
    /**
     * How close the user has to press to the current selection line
     * to hide it. If the press occurs further away it is treated as a
     * new selection.
     *
     * Also determines the minimum movement necessary for a gesture to be
     * recognized as a drag event.
     *
     * Defaults to `horizontalScale(20)`.
     */
    xPressDelta?: number;
    /**
     * Timestamp for which to display a selection line.
     *
     * Timestamps should be given as unix timestamps (i.e. in seconds).
     */
    selectedTimestamp?: number;
    /**
     * Callback that is called when the selected timestamp changes.
     * When deselected, the callback is called with `undefined`.
     *
     * This is a controlled value, changes to the timestamp have to be
     * propagated back via `selectedTimestamp`.
     */
    onSelectionChange?(selectedTimestamp: number | undefined): void;
    /**
     * Callback that is called when user starts or stops dragging the selection line.
     *
     * @param start  `true` on drag start, `false` on drag end
     */
    onDrag?(start: boolean): void;
};
//# sourceMappingURL=common-props.d.ts.map