"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SelectionSvg = void 0;
const react_1 = __importStar(require("react"));
const react_native_svg_1 = require("react-native-svg");
const lodash_1 = require("lodash");
const ui_1 = require("@cs-bosch-medusa/ui");
const theme_1 = require("../../theme");
const utils_1 = require("../../utils");
const DEFAULT_X_PRESS_DELTA = (0, ui_1.horizontalScale)(20);
const SelectionSvg = ({ width, height, children, xScale, xPressDelta = DEFAULT_X_PRESS_DELTA, selectedTimestamp, onDrag, onSelectionChange, testID, }) => {
    const { chartTheme } = (0, theme_1.useChartTheme)();
    // eslint-disable-next-line react/hook-use-state
    const [, setSelectedX] = (0, react_1.useState)();
    const isDrag = (0, react_1.useRef)(false);
    const initialX = (0, react_1.useRef)(0);
    (0, react_1.useEffect)(() => {
        if (selectedTimestamp === undefined) {
            // eslint-disable-next-line unicorn/no-useless-undefined
            setSelectedX(undefined);
        }
    }, [selectedTimestamp]);
    const handleXSelection = (0, react_1.useCallback)((newSelectedX) => {
        const [startTime, endTime] = xScale.domain();
        const timestamp = xScale.invert(newSelectedX).valueOf();
        onSelectionChange((0, lodash_1.clamp)(timestamp, startTime.valueOf(), endTime.valueOf()));
    }, [onSelectionChange, xScale]);
    const onSvgPress = (0, react_1.useMemo)(() => onSelectionChange
        ? (event) => {
            onDrag === null || onDrag === void 0 ? void 0 : onDrag(true);
            const newSelectedX = event.nativeEvent.locationX;
            setSelectedX((prevX) => {
                if (!isDrag.current &&
                    (0, utils_1.isNumber)(prevX) &&
                    Math.abs(newSelectedX - prevX) < xPressDelta) {
                    isDrag.current = false;
                    // eslint-disable-next-line unicorn/no-useless-undefined
                    onSelectionChange(undefined);
                    return;
                }
                isDrag.current = false;
                handleXSelection(newSelectedX);
                return newSelectedX;
            });
        }
        : undefined, [handleXSelection, onDrag, onSelectionChange, xPressDelta]);
    const onTouchMove = (0, react_1.useMemo)(() => onSelectionChange
        ? (event) => {
            onDrag === null || onDrag === void 0 ? void 0 : onDrag(true);
            const newSelectedX = event.nativeEvent.locationX;
            /*
            NOTE: on some devices a simple press is seen as a touch move event,
            so the distance moved has to be checked to distinguish between the two.
            */
            if (Math.abs(initialX.current - newSelectedX) >=
                xPressDelta) {
                isDrag.current = true;
            }
            if (isDrag.current) {
                setSelectedX(newSelectedX);
                handleXSelection(newSelectedX);
            }
        }
        : undefined, [handleXSelection, onDrag, onSelectionChange, xPressDelta]);
    const onTouchStart = (0, react_1.useCallback)((event) => {
        initialX.current = event.nativeEvent.locationX;
    }, []);
    const onTouchEnd = (0, react_1.useCallback)(() => {
        onDrag === null || onDrag === void 0 ? void 0 : onDrag(false);
    }, [onDrag]);
    return (react_1.default.createElement(react_native_svg_1.Svg, { height: height, onPress: onSvgPress, onTouchEnd: onTouchEnd, onTouchMove: onTouchMove, onTouchStart: onTouchStart, style: {
            backgroundColor: chartTheme.doubleAxisChart.backgroundColor,
        }, width: width, ...(0, ui_1.getTestIdProps)(testID) },
        react_1.default.createElement(react_native_svg_1.Rect, { fill: "none", height: height, width: width, x: 0, y: 0 }),
        children));
};
exports.SelectionSvg = SelectionSvg;
