import { Colors, TestProps } from "@cs-bosch-medusa/ui";
export type RealTimeLineChartValues = {
    [key: string]: RealTimeLineChartValue[];
};
export type RealTimeLineChartValue = {
    timestamp: number;
    value: number | string;
};
export type RealTimeLineChartIntervalRange = {
    /** X axis minimum value. */
    xMin: number;
    /** X axis maximum value. */
    xMax: number;
    /** Y axis minimum value. */
    yMin: number;
    /** Y axis maximum value. */
    yMax: number;
};
export type DefaultRealTimeLineChartIntervalRange = Partial<RealTimeLineChartIntervalRange>;
export type RealTimeLineChartProps = TestProps & {
    /** Width of the svg. */
    width: number;
    /** Height of the svg. */
    height: number;
    /**
     * X axis ticks number.
     * At least 2 ticks is required to be given,
     * otherwise 5 ticks are shown.
     */
    xTicksNumber: number;
    /**
     * Y axis ticks number.
     * At least 2 ticks is required to be given,
     * otherwise 5 ticks are shown.
     */
    yTicksNumber: number;
    /** List of parameter ids, which are shown on the line chart. */
    activeParameters: string[];
    /** Formatting function of the x axis timestamp values. */
    formatDate(date: Date): string;
    /** Formatting function of the y values. */
    formatValue(value: number): string;
    /**
     * Each property of the object represents a line within the chart.
     * The line is defined with `{ value, timestamp }` object list.
     * The keys have to match with the parameter ids.
     */
    values: RealTimeLineChartValues;
    /**
     * Each property of the object represents
     * the color of a line within the chart.
     * The keys have to match with the parameter ids.
     */
    colors: {
        [key: string]: Colors;
    };
    /**
     * Initial range to display on the chart's axes when there
     * are not enough data points to calculate the range
     * (at least 2 points needed for one parameter).
     *
     * For limiting the displayed range calculated from the values,
     * take a look at the `interval` and the `defaultIntervalRange` options.
     */
    initialIntervalRange: RealTimeLineChartIntervalRange;
    /**
     * Overrides the range boundaries with the provided values.
     * Any boundary value not specified here will be calculated
     * from the data points given in `values`.
     *
     * __Important notices:__
     * * This option prevents calculating the range based on the given values.
     * * This option ignores the `interval` property.
     */
    defaultIntervalRange?: DefaultRealTimeLineChartIntervalRange;
    /**
     * Show the first and the last point of the lines as circles.
     *
     * Defaults to `true`.
     */
    showPoints?: boolean;
    /**
     * Line chart interval time in milliseconds.
     * This options reduces the shown values to the last interval period.
     *
     * 60000 = 1 minute
     *
     * 0 sets to the `values` interval range.
     *
     * Defaults to `0`.
     *
     * __Important notice:__ If the `defaultIntervalRange` option is given
     * then the interval option is not considered, also the
     * interval range is not calculated from the `values` property.
     */
    interval?: number;
    /**
     * Line chart right side time shift in milliseconds.
     *
     * This option increases the x axis maximum range
     * in order to add a right side padding of the line chart.
     *
     * This option is usually used to give a "live motion monitorization sense".
     *
     * 60000 = 1 minute
     *
     * Defaults to `0`.
     */
    rightSideTimeShift?: number;
    /**
     * Has only effect on the generated base64 content.
     * If value is `true` the generated image from the svg will contain a background
     * otherwise the background is empty. This is the default behavior.
     */
    withBackground?: boolean;
};
//# sourceMappingURL=real-time-line-chart-props.d.ts.map