import React from "react";
import { Moment } from "moment";
import { FlatList } from "react-native";
import { DateWithTimestamp } from "./patched-types";
export type CalendarRef = {
    /**
     * Jumps to the specified date's month
     * and triggers a change event for the specified date.
     */
    jumpTo(date: Date): void;
};
/**
 * Sets up the ref for `Calendar`-reliant components so it provides the
 * desired jump to date functionality.
 *
 * @param ref The reference to set up the imperative methods on.
 * @param selectedDay The currently selected day, formatted as a string.
 * @param onPress Callback to trigger with the date change.
 *
 * @returns A string representation of a date to use as `initialDate` to
 * ensure the correct month will be refocused.
 */
export declare function useCalendarRef(ref: React.Ref<CalendarRef>, selectedDay: string, onPress: (dateToJumpTo: DateWithTimestamp) => void): {
    initialMonth: string;
};
/**
 * Sets up the ref for Calendar List components so it provides the
 * desired jump to date functionality.
 *
 * @param ref The reference to set up the imperative methods on.
 * @param calendarList The data the list is displaying.
 * @param resolution Whether the list shows months or years.
 * @param onChange Callback to trigger with the date change.
 *
 * @returns A ref object to pass to the underlying `FlatList`.
 */
export declare function useCalendarListRef(ref: React.Ref<CalendarRef>, calendarList: {
    date: Moment;
}[], resolution: "month" | "year", onChange: (startDate: Date) => void): React.RefObject<FlatList<any>>;
//# sourceMappingURL=calendar-ref.d.ts.map