package com.bosch.tt.rnmedusa.ui.components;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.common.MapBuilder;
import com.facebook.react.uimanager.SimpleViewManager;
import com.facebook.react.uimanager.ThemedReactContext;
import com.facebook.react.uimanager.annotations.ReactProp;

import java.util.Map;

public class ActivityIndicatorManager extends SimpleViewManager<ActivityIndicator> {
	public static final String REACT_CLASS = "RNMedLoadingIndicator";
	public static final String PROP_BOX_SIZE = "boxSize";
	public static final String PROP_TOP_COLORS = "topColors";
	public static final String PROP_BOTTOM_COLORS = "bottomColors";
	public static final String PROP_MIDDLE_COLORS = "middleColors";

	public static final String COMMAND_START_ANIMATION = "startAnimation";
	public static final String COMMAND_STOP_ANIMATION = "stopAnimation";

	@NonNull
	@Override
	public String getName() {
		return REACT_CLASS;
	}

	@NonNull
	@Override
	protected ActivityIndicator createViewInstance(@NonNull ThemedReactContext reactContext) {
		return new ActivityIndicator(reactContext);
	}

	@ReactProp(name=PROP_BOX_SIZE)
	public void setBoxSize(ActivityIndicator indicator, int boxSize) {
		indicator.setBoxSize(boxSize);
	}

	@ReactProp(name=PROP_TOP_COLORS)
	public void setTopColors(ActivityIndicator indicator, ReadableArray topColors) {
		indicator.setTopColors(topColors);
	}

	@ReactProp(name=PROP_BOTTOM_COLORS)
	public void setBottomColors(ActivityIndicator indicator, ReadableArray bottomColors) {
		indicator.setBottomColors(bottomColors);
	}

	@ReactProp(name=PROP_MIDDLE_COLORS)
	public void setMiddleColors(ActivityIndicator indicator, ReadableArray middleColors) {
		indicator.setMiddleColors(middleColors);
	}

	@Override
	public void receiveCommand(@NonNull ActivityIndicator root, String commandName, @Nullable ReadableArray args) {
		switch (commandName) {
			case COMMAND_START_ANIMATION:
				root.startAnimation();
				break;
			case COMMAND_STOP_ANIMATION:
				root.endAnimation();
				break;
		}
	}
}
