"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CollapsibleComponent = void 0;
/* eslint-disable react/jsx-props-no-spreading */
const react_1 = __importStar(require("react"));
const react_native_1 = require("react-native");
const icons_1 = require("../icons");
const theme_1 = require("../../theme");
const util_1 = require("../../util");
const styles = react_native_1.StyleSheet.create({
    centered: {
        alignItems: "center",
    },
    collapsible: {
        position: "absolute",
        width: "100%",
    },
    container: {
        overflow: "hidden",
    },
});
/**
 * Contains the animation logic for the height of the collapsible content
 * and the scale of the DownIcon. Returns the height, the scale and the toggle function.
 *
 * @param contentHeight  The height of the collapsible content
 * @param openInitially  Whether to show the collapsible content on initial render
 */
const useCollapseAnimation = (contentHeight, openInitially = false) => {
    const [collapsed, setCollapsed] = (0, react_1.useState)(!openInitially);
    const height = (0, react_1.useRef)(new react_native_1.Animated.Value(0));
    const scale = (0, react_1.useRef)(new react_native_1.Animated.Value(openInitially ? 0 : 1));
    // We need this to support content with dynamic height
    (0, react_1.useEffect)(() => {
        if (!collapsed) {
            react_native_1.Animated.timing(height.current, {
                toValue: contentHeight,
                duration: theme_1.ANIMATION_DURATION.COLLAPSE,
                useNativeDriver: false,
            }).start();
        }
    }, [collapsed, contentHeight]);
    const toggle = (0, react_1.useCallback)(() => {
        setCollapsed((wasCollapsed) => {
            // We set the initial height of the animated view, this also cancels out any ongoing animation
            height.current.setValue(wasCollapsed ? 0 : contentHeight);
            react_native_1.Animated.timing(height.current, {
                // The final height the animated view has to reach
                toValue: wasCollapsed ? contentHeight : 0,
                duration: theme_1.ANIMATION_DURATION.COLLAPSE,
                useNativeDriver: false,
            }).start();
            scale.current.setValue(wasCollapsed ? 1 : 0);
            react_native_1.Animated.timing(scale.current, {
                toValue: wasCollapsed ? 0 : 1,
                duration: theme_1.ANIMATION_DURATION.COLLAPSE,
                useNativeDriver: true,
            }).start();
            return !wasCollapsed;
        });
    }, [contentHeight]);
    const collapse = (0, react_1.useCallback)(() => {
        if (!collapsed) {
            toggle();
        }
    }, [collapsed, toggle]);
    const open = (0, react_1.useCallback)(() => {
        if (collapsed) {
            toggle();
        }
    }, [collapsed, toggle]);
    return { toggle, open, collapse, height, scale };
};
/**
 * A component that handles the collapse animation and gives us
 * an easy way to create custom collapsible components.
 */
exports.CollapsibleComponent = (0, react_1.forwardRef)(({ growUpwards = false, render, openInitially, testID }, ref) => {
    const { theme } = (0, theme_1.useTheme)();
    const [contentHeight, setContentHeight] = (0, react_1.useState)(0);
    const collapseAnimation = useCollapseAnimation(contentHeight, openInitially);
    (0, react_1.useImperativeHandle)(ref, () => ({
        open: collapseAnimation.open,
        collapse: collapseAnimation.collapse,
    }));
    const onLayout = (0, react_1.useCallback)((event) => {
        /*
        Rounding is required to not generate many state updates due to floating point
        imprecisions. See #260.
        */
        const roundedHeight = Math.round(event.nativeEvent.layout.height);
        setContentHeight(roundedHeight);
    }, []);
    const CollapsibleContent = (0, react_1.useMemo)(() => 
    // eslint-disable-next-line react/no-unstable-nested-components
    (0, react_1.memo)(({ children }) => (react_1.default.createElement(react_native_1.Animated.View, { style: [
            styles.container,
            { height: collapseAnimation.height.current },
        ], ...(0, util_1.getTestIdProps)(`${testID}.content`) },
        react_1.default.createElement(react_native_1.View, { onLayout: onLayout, style: styles.collapsible }, children)))), [collapseAnimation.height, onLayout, testID]);
    const CollapseIcon = (0, react_1.useMemo)(() => 
    // eslint-disable-next-line react/no-unstable-nested-components
    (0, react_1.memo)(({ pressable = true, ...iconProps }) => (react_1.default.createElement(react_native_1.Animated.View, { style: {
            transform: [
                {
                    scaleY: collapseAnimation.scale.current.interpolate({
                        inputRange: [0, 1],
                        outputRange: growUpwards
                            ? [1, -1]
                            : [-1, 1],
                    }),
                },
            ],
        } },
        react_1.default.createElement(react_native_1.TouchableOpacity, { activeOpacity: theme.activeOpacity, disabled: !pressable, onPress: pressable ? collapseAnimation.toggle : undefined, style: styles.centered, ...(0, util_1.getTestIdProps)(`${testID}.toggle`) },
            react_1.default.createElement(icons_1.DownIcon, { ...iconProps }))))), [
        collapseAnimation.scale,
        collapseAnimation.toggle,
        growUpwards,
        testID,
        theme.activeOpacity,
    ]);
    return render(CollapsibleContent, CollapseIcon, collapseAnimation.toggle);
});
