"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NumberInputField = void 0;
const react_1 = __importStar(require("react"));
const react_native_1 = require("react-native");
const lodash_1 = require("lodash");
const theme_1 = require("../../theme");
const text_field_validation_info_1 = require("./text-field-validation-info");
const text_field_style_1 = require("./text-field-style");
const input_field_1 = require("./input-field");
const icons_1 = require("../icons");
const text_field_focus_hook_1 = require("./text-field-focus-hook");
const util_1 = require("../../util");
const text_field_ref_1 = require("./text-field-ref");
const SYMBOL = {
    HYPHEN: "-",
    PERIOD: ".",
};
/**
 * Finds all PERIOD characters in the text.
 */
const PERIOD_REG_EXP = new RegExp(`\\${SYMBOL.PERIOD}`, "g");
/**
 * Finds all non-digit characters, except the HYPHEN at the beginning of the text.
 */
const NON_DIGITAL_REG_EXP = new RegExp(`(?!^${SYMBOL.HYPHEN})[^\\d${SYMBOL.PERIOD}]`, "g");
const styles = react_native_1.StyleSheet.create({
    button: {
        alignItems: "center",
        alignSelf: "stretch",
        justifyContent: "center",
        padding: theme_1.METRICS.CONTAINER.basePadding +
            theme_1.METRICS.CONTAINER.halfBasePadding / 2,
    },
    flex: {
        flex: 1,
    },
    rowContainer: {
        alignItems: "center",
        flexDirection: "row",
    },
});
exports.NumberInputField = (0, react_1.forwardRef)(({ decimalSeparator = SYMBOL.PERIOD, disabled, label, precision = 0, step = 1, min, max, showButtons = true, onBlur, onFocus, onValueChange, validationResult, validationMessage, value, textAlign, testID, }, ref) => {
    const { theme } = (0, theme_1.useTheme)();
    const inputRef = (0, text_field_ref_1.useTextFieldRef)(ref);
    const [displayValue, setDisplayValue] = (0, react_1.useState)((0, lodash_1.round)(value, precision).toString());
    const { focused, onTextFieldFocus, onTextFieldBlur } = (0, text_field_focus_hook_1.useTextFieldFocus)(onFocus, onBlur);
    const { fillColor, textColor } = (0, text_field_style_1.useTextFieldStyle)({
        focused,
        disabled,
    });
    const decrementValue = (0, react_1.useCallback)(() => {
        const decValue = clamp(value - step, min, max);
        const decRounded = (0, lodash_1.round)(decValue, precision);
        onValueChange(decRounded);
    }, [max, min, onValueChange, precision, step, value]);
    const incrementValue = (0, react_1.useCallback)(() => {
        const incValue = clamp(value + step, min, max);
        const incRounded = (0, lodash_1.round)(incValue, precision);
        onValueChange(incRounded);
    }, [max, min, onValueChange, precision, step, value]);
    const onChange = (0, react_1.useCallback)((res) => {
        let text = res.replace(decimalSeparator, SYMBOL.PERIOD);
        if (text.length === 0 ||
            (text.length === 1 && text[0] === SYMBOL.HYPHEN) ||
            text.indexOf(SYMBOL.PERIOD) === text.length - 1) {
            setDisplayValue(text.replace(SYMBOL.PERIOD, decimalSeparator));
            return;
        }
        let convertedText = text
            .replace(PERIOD_REG_EXP, (dot, index, text) => {
            const isFirstDot = text.indexOf(dot) === index;
            return isFirstDot ? dot : "";
        })
            .replace(NON_DIGITAL_REG_EXP, "");
        if (isPrecisionLimitExceeded(convertedText, precision)) {
            convertedText = truncToPrecision(convertedText, precision);
        }
        setDisplayValue(convertedText.replace(SYMBOL.PERIOD, decimalSeparator));
        const value = Number.parseFloat(convertedText);
        if (!Number.isNaN(value) &&
            value.toString() === convertedText) {
            onValueChange(value);
        }
    }, [decimalSeparator, onValueChange, precision]);
    const onBlurInternal = () => {
        const displayedNumber = Number.parseFloat(displayValue.replace(decimalSeparator, SYMBOL.PERIOD));
        if (displayValue.includes(decimalSeparator) &&
            displayValue.length !== 1 &&
            displayValue.indexOf(decimalSeparator) ===
                displayValue.length - 1) {
            onValueChange(clamp(displayedNumber, min, max));
            /**
             * We set the displayValue because useEffect will not fire in every case.
             * e.g.: `value` is initially 0 and we delete the value and input
             * a decimal separator and trigger onBlur.
             */
            setDisplayValue((0, lodash_1.round)(value, precision)
                .toString()
                .replace(SYMBOL.PERIOD, decimalSeparator));
        }
        else if (Number.isNaN(displayedNumber)) {
            onValueChange(0);
            setDisplayValue("0");
        }
        else if (clamp(displayedNumber, min, max) !== displayedNumber ||
            displayedNumber.toString() !== displayValue) {
            const clampedNumber = clamp(displayedNumber, min, max);
            onValueChange(clampedNumber);
            setDisplayValue(clampedNumber
                .toString()
                .replace(SYMBOL.PERIOD, decimalSeparator));
        }
        onTextFieldBlur();
    };
    (0, react_1.useEffect)(() => {
        setDisplayValue((0, lodash_1.round)(value, precision)
            .toString()
            .replace(SYMBOL.PERIOD, decimalSeparator));
    }, [value, precision, decimalSeparator]);
    const onBoxPress = (0, react_1.useCallback)(() => {
        var _a;
        (_a = inputRef.current) === null || _a === void 0 ? void 0 : _a.focus();
    }, [inputRef]);
    const decrementDisabled = disabled || minReached(value, min);
    const incrementDisabled = disabled || maxReached(value, max);
    return (react_1.default.createElement(react_native_1.View, { style: text_field_style_1.textFieldStyle.container },
        react_1.default.createElement(react_native_1.View, { style: styles.rowContainer },
            react_1.default.createElement(react_native_1.Pressable, { onPress: onBoxPress, style: [
                    text_field_style_1.textFieldStyle.textContainer,
                    { backgroundColor: fillColor },
                    styles.flex,
                ], ...(0, util_1.getTestIdProps)(`${testID}.box`) },
                label && (react_1.default.createElement(react_native_1.Text, { allowFontScaling: theme.allowFontScaling, numberOfLines: 1, style: [
                        text_field_style_1.textFieldStyle.label,
                        {
                            color: textColor,
                            fontFamily: theme.regularFont,
                        },
                    ] }, label)),
                react_1.default.createElement(input_field_1.InputField
                /** AutoCapitalize is set to "words" to support decimal precision of the
                 * number pad for devices running an older version of Android.
                 * See: https://github.com/facebook/react-native/issues/12988 */
                , { 
                    /** AutoCapitalize is set to "words" to support decimal precision of the
                     * number pad for devices running an older version of Android.
                     * See: https://github.com/facebook/react-native/issues/12988 */
                    autoCapitalize: "words", disabled: disabled, keyboardType: react_native_1.Platform.select({
                        android: "number-pad",
                        ios: "numbers-and-punctuation",
                    }), 
                    // eslint-disable-next-line react/jsx-no-bind
                    onBlur: onBlurInternal, onFocus: onTextFieldFocus, onValueChange: onChange, ref: inputRef, style: label
                        ? text_field_style_1.textFieldStyle.textFieldWithLabel
                        : text_field_style_1.textFieldStyle.textField, testID: testID, textAlign: textAlign, value: displayValue })),
            showButtons && (react_1.default.createElement(NumberInputButton, { disabled: decrementDisabled, icon: icons_1.LessMinimizeIcon, onPress: decrementValue, testID: `${testID}.subtract` })),
            showButtons && (react_1.default.createElement(NumberInputButton, { disabled: incrementDisabled, icon: icons_1.AddIcon, onPress: incrementValue, testID: `${testID}.add` }))),
        react_1.default.createElement(text_field_validation_info_1.TextFieldValidationInfo, { disabled: disabled, focused: focused, testID: `${testID}.validation`, validationMessage: validationMessage, validationResult: validationResult })));
});
const NumberInputButton = ({ disabled, onPress, icon: Icon, testID, }) => {
    const { theme } = (0, theme_1.useTheme)();
    const getStyle = (0, react_1.useCallback)(({ pressed }) => [
        styles.button,
        {
            backgroundColor: getButtonStyle(theme, { pressed, disabled })
                .fillColor,
        },
    ], [theme, disabled]);
    return (react_1.default.createElement(react_native_1.Pressable, { disabled: disabled, onPress: onPress, style: getStyle, ...(0, util_1.getTestIdProps)(testID) }, ({ pressed }) => {
        const color = getButtonStyle(theme, {
            pressed,
            disabled,
        }).textColor;
        return react_1.default.createElement(Icon, { fillColor: color, iconSize: icons_1.IconSize.XSMALL });
    }));
};
const getButtonStyle = (theme, { pressed, disabled }) => {
    if (disabled) {
        return theme.inputs.disabled;
    }
    return pressed ? theme.inputs.pressed : theme.inputs.default;
};
const clamp = (value, min, max) => {
    return clampMax(clampMin(value, min), max);
};
const clampMin = (value, min) => {
    if (min === undefined) {
        return value;
    }
    return Math.max(value, min);
};
const clampMax = (value, max) => {
    if (max === undefined) {
        return value;
    }
    return Math.min(value, max);
};
const minReached = (value, min) => {
    if (min === undefined) {
        return false;
    }
    return value <= min;
};
const maxReached = (value, max) => {
    if (max === undefined) {
        return false;
    }
    return value >= max;
};
const isPrecisionLimitExceeded = (value, precision) => {
    const values = value.split(SYMBOL.PERIOD, 2);
    return values.length !== 1 && values[1].length > precision;
};
const truncToPrecision = (value, precision) => {
    const values = value.split(SYMBOL.PERIOD, 2);
    return `${values[0]}.${values[1].slice(0, Math.max(0, precision))}`;
};
