"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AndroidModal = void 0;
/* eslint-disable react/jsx-props-no-spreading */
const react_1 = __importDefault(require("react"));
const react_native_1 = require("react-native");
const theme_1 = require("../../theme");
const util_1 = require("../../util");
const styles = react_native_1.StyleSheet.create({
    button: {
        paddingHorizontal: theme_1.METRICS.CONTAINER.basePadding,
    },
    buttonContainer: {
        flexDirection: "row",
        justifyContent: "flex-end",
    },
    buttonText: {
        fontFamily: "sans-serif-medium",
        fontSize: theme_1.FontSize.SMALL,
        textTransform: "uppercase",
    },
    container: {
        maxHeight: "80%",
        maxWidth: "80%",
        padding: theme_1.METRICS.CONTAINER.basePadding,
    },
    content: {
        padding: theme_1.METRICS.CONTAINER.basePadding,
        paddingBottom: 0,
    },
    leftButtonContainer: {
        flex: 1,
    },
    message: {
        fontFamily: "sans-serif",
        fontSize: theme_1.FontSize.SMALL,
        paddingBottom: theme_1.METRICS.CONTAINER.basePadding,
    },
    modal: {
        alignItems: "center",
        flex: 1,
        justifyContent: "center",
    },
    title: {
        fontFamily: "sans-serif-medium",
        fontSize: theme_1.FontSize.MEDIUM,
        paddingBottom: theme_1.METRICS.CONTAINER.basePadding,
    },
    view: {
        paddingBottom: theme_1.METRICS.CONTAINER.basePadding,
    },
});
const AndroidButton = ({ text, onPress, testID }) => {
    const { theme } = (0, theme_1.useTheme)();
    return (react_1.default.createElement(react_native_1.TouchableOpacity, { onPress: onPress, style: styles.button, ...(0, util_1.getTestIdProps)(testID) },
        react_1.default.createElement(react_native_1.Text, { style: [styles.buttonText, { color: theme.primaryColor }] }, text)));
};
const AndroidButtonSection = ({ buttons }) => {
    if (buttons.length > 3) {
        console.warn("The modal supports max 3 buttons on android. If you need more, please create a custom view.");
    }
    return buttons.length >= 3 ? (react_1.default.createElement(react_native_1.View, { style: styles.buttonContainer },
        react_1.default.createElement(react_native_1.View, { style: styles.leftButtonContainer },
            react_1.default.createElement(AndroidButton, { ...buttons[0] })),
        react_1.default.createElement(AndroidButton, { ...buttons[1] }),
        react_1.default.createElement(AndroidButton, { ...buttons[2] }))) : (react_1.default.createElement(react_native_1.View, { style: styles.buttonContainer }, buttons.map((buttonProps) => (react_1.default.createElement(AndroidButton, { ...buttonProps, key: buttonProps.text })))));
};
const AndroidModal = ({ visible, transparent = true, title, message, buttons, children, testID, }) => {
    const { theme } = (0, theme_1.useTheme)();
    return (react_1.default.createElement(react_native_1.Modal, { transparent: transparent, visible: visible, ...(0, util_1.getTestIdProps)(`${testID}.modal`) },
        react_1.default.createElement(react_native_1.View, { style: [
                styles.modal,
                { backgroundColor: theme.modalBackgroundColor },
            ] },
            react_1.default.createElement(react_native_1.View, { style: [
                    styles.container,
                    {
                        backgroundColor: theme.backgroundColor,
                    },
                ], ...(0, util_1.getTestIdProps)(testID) },
                react_1.default.createElement(react_native_1.View, { style: styles.content },
                    title !== undefined && (react_1.default.createElement(react_native_1.Text, { style: [
                            styles.title,
                            { color: theme.textColor },
                        ] }, title)),
                    message !== undefined && (react_1.default.createElement(react_native_1.Text, { style: [
                            styles.message,
                            { color: theme.textColor },
                        ] }, message)),
                    children && (react_1.default.createElement(react_native_1.View, { style: styles.view }, children))),
                buttons && react_1.default.createElement(AndroidButtonSection, { buttons: buttons })))));
};
exports.AndroidModal = AndroidModal;
