"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RadioButton = void 0;
const react_1 = __importDefault(require("react"));
const react_native_1 = require("react-native");
const theme_1 = require("../../theme");
const label_1 = require("../text/label");
const selection_circle_1 = require("./selection-circle");
const separators_1 = require("../separators");
const styles = react_native_1.StyleSheet.create({
    circle: {
        marginRight: theme_1.METRICS.CONTAINER.baseMargin,
    },
    flex: {
        flex: 1,
    },
    radioButtonWrapper: {
        alignItems: "center",
        alignSelf: "baseline",
        flexDirection: "row",
        padding: theme_1.METRICS.CONTAINER.basePadding,
    },
});
/**
 * Component to be used inside RadioButtonContainer to list the available options.
 */
const RadioButton = (props) => {
    const { theme } = (0, theme_1.useTheme)();
    const { disabled = false, label, selected, pressed, children, separator = false, style, testID, } = props;
    const labelColor = !disabled ? theme.textColor : theme.disabledColor;
    const rowStyle = react_native_1.StyleSheet.flatten([styles.radioButtonWrapper, style]);
    return (react_1.default.createElement(react_1.default.Fragment, null,
        react_1.default.createElement(react_native_1.View, { style: rowStyle },
            react_1.default.createElement(selection_circle_1.SelectionCircle, { disabled: disabled, pressed: pressed, selected: selected, style: styles.circle, testID: testID }),
            label && (react_1.default.createElement(label_1.Label, { style: [{ color: labelColor }, styles.flex] }, label)),
            !label && children && (react_1.default.createElement(react_native_1.View, { style: styles.flex }, children))),
        separator && react_1.default.createElement(separators_1.Separator, null)));
};
exports.RadioButton = RadioButton;
