import { FC, PropsWithChildren } from "react";
import { Theme } from "./themes";
interface ThemeResult<T extends Theme> {
    theme: T;
    setTheme(theme: T): void;
}
type Props<T extends Theme> = {
    initialTheme?: T;
};
export declare const ThemeProvider: FC<PropsWithChildren<Props<Theme>>>;
/**
 * If the `useTheme` is called without a type then the hook will work as *before*.
 * If a type is passed down for the hook, then the hook will return an **object** and a
 * **setter method** with the same types.
 * > OBS: the type must extend the original `Theme` interface.
 * @example
 *
 * const [theme, setTheme] = useTheme<ExtendedTheme>();
 * // set the new extended theme
 * setTheme({ ...boschTheme, highlightText: Colors.BOSCH_RED_50 });
 *
 * // use with confidence
 * const colorOfText: Colors = theme.highlightText;
 */
export declare function useTheme<T extends Theme = Theme>(): ThemeResult<T>;
export {};
//# sourceMappingURL=theme-context.d.ts.map