//
//  ProviderManager.h
//  RNPrivacyComponent
//
//  Created by István Bege on 31/10/2018.
//  Copyright (C) 2018, Bosch Thermotechnik GmbH. All rights reserved.
//

//  Singleton class which holds the necessary TTPPrivacyProvider and TTActivityProvider object instances and country objects.

#import <Foundation/Foundation.h>

#import <TTPrivacyComponent/TTPPrivacyProvider.h>
#import <TTPrivacyComponent/TTPPreviewValueProvider.h>
#import <TTPrivacyComponent/TTPProductionValueProvider.h>
#import <TTPrivacyComponent/TTPActivityProvider.h>

@interface ProviderManager : NSObject {
    TTPPrivacyProvider* privacyProvider;
    TTPPrivacyProvider* termsOfUseProvider;
    TTPActivityProvider* activityProvider;
    TTPCountry* selectedCountry;
    NSArray<TTPCountry*>* supportedCountries;
}

@property (nonatomic, readonly) TTPPrivacyProvider* privacyProvider;
@property (nonatomic, readonly) TTPPrivacyProvider* termsOfUseProvider;
@property (nonatomic, readonly) TTPActivityProvider* activityProvider;
@property (nonatomic, retain) TTPCountry* selectedCountry;
@property (nonatomic, retain) NSArray<TTPCountry*>* supportedCountries;

+ (ProviderManager*)getInstance;
- (void)initPrivacyProviderWithLeanIX:(nonnull NSString*)leanIX productionMode:(BOOL)productionMode;
- (void)initTermsOfUseProvider;
- (void)initActivityConsentProvider:(nonnull NSString*)activityId;

@end
