//
//  RNPrivacyComponent.m
//  RNPrivacyComponent
//
//  Created by István Bege on 26/10/2018.
//  Copyright (C) 2018, Bosch Thermotechnik GmbH. All rights reserved.
//

#import <React/RCTComponent.h>
#import <TTPrivacyComponent/TTPMetaDocument.h>

#import "RNPrivacyComponent.h"
#import "ProviderManager.h"
#import "Assembler.h"
#import "Constants.h"

@interface RNPrivacyComponent ()

@property (readonly) ProviderManager* providerManager;
@property (readonly) NSDateFormatter* dateFormatter;

@end

@implementation RNPrivacyComponent

- (dispatch_queue_t)methodQueue
{
    return dispatch_get_main_queue();
}

RCT_EXPORT_MODULE()

# pragma mark Country specific methods

RCT_EXPORT_METHOD(setSupportedCountries:(NSArray<NSDictionary*>*)supportedCountries reactTag:(nonnull NSNumber *)reactTag)
{
    [_providerManager setSupportedCountries:[Assembler createCountryArray:supportedCountries]];
}

RCT_EXPORT_METHOD(setSelectedCountry:(NSDictionary*)rawCountry reactTag:(nonnull NSNumber *)reactTag)
{
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        [self->_providerManager setSelectedCountry:[Assembler createCountry:rawCountry]];
    });
}

# pragma mark Privacy Policy specific methods

RCT_EXPORT_METHOD(initPrivacyProvider:(NSString*)leanIX productionMode:(BOOL)productionMode)
{
    _providerManager = [ProviderManager getInstance];
    [_providerManager initPrivacyProviderWithLeanIX:leanIX productionMode:productionMode];
    _dateFormatter = [[NSDateFormatter alloc] init];
    [_dateFormatter setDateFormat:@"yyyy-MM-dd'T'HH:mm:ss"];
}

RCT_EXPORT_METHOD(acceptPrivacyPolicy:(NSString*)documentId)
{
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        [[self->_providerManager privacyProvider] accept:documentId];
    });
}

RCT_EXPORT_METHOD(isPrivacyPolicyAccepted:(NSString*)documentId resolver:(RCTPromiseResolveBlock)resolve rejecter:(RCTPromiseRejectBlock)reject)
{
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        BOOL result = [[self->_providerManager privacyProvider] isAccepted:documentId];
        resolve([NSNumber numberWithBool:result]);
    });
}

RCT_EXPORT_METHOD(getAvailablePrivacyPolicyDocuments:(RCTPromiseResolveBlock)resolve rejecter:(RCTPromiseRejectBlock)reject)
{
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        [self getAvailableDocumentWithProvider:[self->_providerManager privacyProvider] resolver:resolve rejecter:reject];
    });
}

RCT_EXPORT_METHOD(getPrivacyPolicyLastCheckedTimestamp:(NSString*)country language:(NSString*)language resolver:(RCTPromiseResolveBlock)resolve rejecter:(RCTPromiseRejectBlock)reject)
{
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        NSDate* result = [[self->_providerManager privacyProvider] lastCheckedTimestampWithCountry:country language:language];
        if (result != nil) {
            resolve([[NSString alloc] initWithString:[self->_dateFormatter stringFromDate:result]]);
        } else {
            reject(@"404", @"The document was not checked on the required country and language.", nil);
        }
    });
}

RCT_EXPORT_METHOD(getTimestampOfPrivacyPolicyConsent:(NSString*)documentId resolver:(RCTPromiseResolveBlock)resolve rejecter:(RCTPromiseRejectBlock)reject)
{
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        NSDate* result = [[self->_providerManager privacyProvider] getTimestampOfConsent:documentId];
        if (result != nil) {
            resolve([[NSString alloc] initWithString:[self->_dateFormatter stringFromDate:result]]);
        } else {
            reject(@"404", @"The provided document was not accepted.", nil);
        }
    });
}

RCT_EXPORT_METHOD(getPrivacyPolicyDocument:(NSString*)country language:(NSString*)language resolver:(RCTPromiseResolveBlock)resolve rejecter:(RCTPromiseRejectBlock)reject)
{
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        [self getDocumentWithProvider:[self->_providerManager privacyProvider] country:country language:language resolver:resolve rejecter:reject];
    });
}

RCT_EXPORT_METHOD(getAvailablePrivacyPolicyCountries:(NSArray<NSDictionary*>*) countriesFromApp resolver:(RCTPromiseResolveBlock)resolve rejecter:(RCTPromiseRejectBlock)reject)
{
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        NSArray<TTPCountry*>* availableCountries = [[self->_providerManager privacyProvider] getAvailableCountries:[Assembler createCountryArray:countriesFromApp]];
        resolve([Assembler createCountryDTOArray:availableCountries]);
    });
}

# pragma mark Terms of Use specific methods

RCT_EXPORT_METHOD(initTermsOfUseProvider)
{
    _providerManager = [ProviderManager getInstance];
    [_providerManager initTermsOfUseProvider];
}

RCT_EXPORT_METHOD(acceptTermsOfUse:(NSString*)documentId)
{
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        [[self->_providerManager termsOfUseProvider] accept:documentId];
    });
}

RCT_EXPORT_METHOD(isTermsOfUseAccepted:(NSString*)documentId resolver:(RCTPromiseResolveBlock)resolve rejecter:(RCTPromiseRejectBlock)reject)
{
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        BOOL result = [[self->_providerManager termsOfUseProvider] isAccepted:documentId];
        resolve([NSNumber numberWithBool:result]);
    });
}

RCT_EXPORT_METHOD(getTermsOfUseDocument:(NSString*)country language:(NSString*)language resolver:(RCTPromiseResolveBlock)resolve rejecter:(RCTPromiseRejectBlock)reject)
{
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        [self getDocumentWithProvider:[self->_providerManager termsOfUseProvider] country:country language:language resolver:resolve rejecter:reject];
    });
}

RCT_EXPORT_METHOD(getAvailableTermsOfUseDocuments:(RCTPromiseResolveBlock)resolve rejecter:(RCTPromiseRejectBlock)reject)
{
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        [self getAvailableDocumentWithProvider:[self->_providerManager termsOfUseProvider] resolver:resolve rejecter:reject];
    });
}

RCT_EXPORT_METHOD(getTermsOfUseLastCheckedTimestamp:(NSString*)country language:(NSString*)language resolver:(RCTPromiseResolveBlock)resolve rejecter:(RCTPromiseRejectBlock)reject)
{
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        NSDate* result = [[self->_providerManager termsOfUseProvider] lastCheckedTimestampWithCountry:country language:language];
        if (result != nil) {
            resolve([[NSString alloc] initWithString:[self->_dateFormatter stringFromDate:result]]);
        } else {
            reject(@"404", @"The document was not checked on the required country and language.", nil);
        }
    });
}

RCT_EXPORT_METHOD(getTimestampOfTermsOfUseConsent:(NSString*)documentId resolver:(RCTPromiseResolveBlock)resolve rejecter:(RCTPromiseRejectBlock)reject)
{
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        NSDate* result = [[self->_providerManager termsOfUseProvider] getTimestampOfConsent:documentId];
        if (result != nil) {
            resolve([[NSString alloc] initWithString:[self->_dateFormatter stringFromDate:result]]);
        } else {
            reject(@"404", @"The provided document was not accepted.", nil);
        }
    });
}

RCT_EXPORT_METHOD(getAvailableTermsOfUseCountries:(NSArray<NSDictionary*>*) countriesFromApp resolver:(RCTPromiseResolveBlock)resolve rejecter:(RCTPromiseRejectBlock)reject)
{
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        NSArray<TTPCountry*>* availableCountries = [[self->_providerManager termsOfUseProvider] getAvailableCountries:[Assembler createCountryArray:countriesFromApp]];
        resolve([Assembler createCountryDTOArray:availableCountries]);
    });
}

# pragma mark Activity Consent specific methods

RCT_EXPORT_METHOD(initActivityConsentProvider:(NSString*)activityId)
{
    [_providerManager initActivityConsentProvider:activityId];
}

RCT_EXPORT_METHOD(acceptActivityConsent:(NSString*)documentId)
{
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        [[self->_providerManager activityProvider] accept:documentId];
    });
}

RCT_EXPORT_METHOD(isActivityConsentAccepted:(NSString*)documentId resolver:(RCTPromiseResolveBlock)resolve rejecter:(RCTPromiseRejectBlock)reject)
{
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        BOOL result = [[self->_providerManager activityProvider] isAccepted:documentId];
        resolve([NSNumber numberWithBool:result]);
    });
}

RCT_EXPORT_METHOD(getActivityConsentDocument:(NSString*)country language:(NSString*)language resolver:(RCTPromiseResolveBlock)resolve rejecter:(RCTPromiseRejectBlock)reject)
{
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        TTPActivityDocument* document = [[self->_providerManager activityProvider] getDocumentWithCountry:country language:language];
        if (document != nil) {
            resolve([Assembler createActivityDocumentDTO:document]);
        } else {
            reject(@"404", @"No activity document found!", nil);
        }
    });
}

RCT_EXPORT_METHOD(getAvailableActivityConsentDocuments:(RCTPromiseResolveBlock)resolve rejecter:(RCTPromiseRejectBlock)reject)
{
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        NSArray<TTPMetaActivityDocument*>* documents = [[self->_providerManager activityProvider] getAvailableDocuments];
        if (documents != nil) {
            resolve([Assembler createActivityDocumentDTOArray:documents]);
        } else {
            reject(@"404", @"No available activity documents!", nil);
        }
    });
}

RCT_EXPORT_METHOD(getActivityConsentLastCheckedTimestamp:(NSString*)country language:(NSString*)language resolver:(RCTPromiseResolveBlock)resolve rejecter:(RCTPromiseRejectBlock)reject)
{
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        NSDate* result = [[self->_providerManager activityProvider] lastCheckedTimestampWithCountry:country language:language];
        if (result != nil) {
            resolve([[NSString alloc] initWithString:[self->_dateFormatter stringFromDate:result]]);
        } else {
            reject(@"404", @"The activity document was not checked on the required country and language.", nil);
        }
    });
}

RCT_EXPORT_METHOD(getTimestampOfActivityConsent:(NSString*)documentId resolver:(RCTPromiseResolveBlock)resolve rejecter:(RCTPromiseRejectBlock)reject)
{
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        NSDate* result = [[self->_providerManager activityProvider] getTimestampOfConsent:documentId];
        if (result != nil) {
            resolve([[NSString alloc] initWithString:[self->_dateFormatter stringFromDate:result]]);
        } else {
            reject(@"404", @"The provided document was not accepted.", nil);
        }
    });
}

RCT_EXPORT_METHOD(getAvailableActivityConsentCountries:(NSArray<NSDictionary*>*) countriesFromApp resolver:(RCTPromiseResolveBlock)resolve rejecter:(RCTPromiseRejectBlock)reject)
{
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        NSArray<TTPCountry*>* availableCountries = [[self->_providerManager activityProvider] getAvailableCountries:[Assembler createCountryArray:countriesFromApp]];
        resolve([Assembler createCountryDTOArray:availableCountries]);
    });
}


# pragma mark private methods

- (void)getDocumentWithProvider:(TTPPrivacyProvider*)provider country:(NSString*)country language:(NSString*)language resolver:(RCTPromiseResolveBlock)resolve rejecter:(RCTPromiseRejectBlock)reject
{
    TTPMetaDocument* document = [provider getDocumentWithCountry:country language:language];
    if (document != nil) {
        NSDictionary *documentDTO = [Assembler createDocumentDTO:document];
        if ([documentDTO count] > 0) {
            resolve(documentDTO);
        } else {
            reject(ERROR_CODE_STR, ERROR_FILE_NOT_FOUND_MSG, nil);
        }
    } else {
        reject(ERROR_CODE_STR, ERROR_FILE_NOT_FOUND_MSG, nil);
    }
}

- (void)getAvailableDocumentWithProvider:(TTPPrivacyProvider*)provider resolver:(RCTPromiseResolveBlock)resolve rejecter:(RCTPromiseRejectBlock)reject
{
    NSArray<TTPMetaDocument*>* documents = [provider getAvailableDocuments];
    if (documents != nil) {
        resolve([Assembler createDocumentDTOArray:documents]);
    } else {
        reject(@"404", @"No available documents!", nil);
    }
}

@end
